@@ -11,7 +11,6 @@ use std::{env, path::PathBuf};
 #[derive(Debug)]
 pub enum Error {
     /// Failed to [sync](`dep::sync`) dependencies.
-    #[cfg(feature = "build")]
     SyncDep(anyhow::Error),
     /// Failed to build *libui*.
     #[cfg(feature = "build")]
@@ -28,18 +27,18 @@ fn main() -> Result<(), Error> {
     let meson_dir = out_dir.join("meson");
     let ninja_dir = out_dir.join("ninja");
 
+    // Cargo will prevent this crate from being published if the build script modifies files
+    // outside `$OUT_DIR` during its operation. To work around this for the purpose of building
+    // *libui*, we copy all non-Rust build dependencies to `$OUT_DIR`.
+    dep::sync("libui-ng", &libui_dir).map_err(Error::SyncDep)?;
+    dep::sync("meson", &meson_dir).map_err(Error::SyncDep)?;
+
+    if build_cfg!(target_os = "linux") {
+        dep::sync("ninja", &ninja_dir).map_err(Error::SyncDep)?;
+    }
+
     #[cfg(feature = "build")]
     {
-        // Cargo will prevent this crate from being published if the build script modifies files
-        // outside `$OUT_DIR` during its operation. To work around this for the purpose of building
-        // *libui*, we copy all non-Rust build dependencies to `$OUT_DIR`.
-        dep::sync("libui-ng", &libui_dir).map_err(Error::SyncDep)?;
-        dep::sync("meson", &meson_dir).map_err(Error::SyncDep)?;
-
-        if build_cfg!(target_os = "linux") {
-            dep::sync("ninja", &ninja_dir).map_err(Error::SyncDep)?;
-        }
-
         libui::build(&libui_dir, &meson_dir, &ninja_dir).map_err(Error::BuildLibui)?;
 
         // Tell Cargo where to find the copy of *libui* that we just built.
@@ -47,10 +46,23 @@ fn main() -> Result<(), Error> {
             "cargo:rustc-link-search={}",
             libui_dir.join("build/meson-out/").display(),
         );
+
+        // Because we are building *libui* from scratch and placing it in `$OUT_DIR`, it makes sense
+        // to link statically. Consequently, as static libraries *do not* contain information on the
+        // shared objects that must be imported, we must tell Cargo (and, by extension, the dynamic
+        // linker) which shared objects we need.
+        import_dylibs();
     }
 
-    // Instruct Cargo to link---either statically or dynamically---to *libui*.
-    println!("cargo:rustc-link-lib=ui");
+    // Instruct Cargo to link to *libui*.
+    println!(
+        "cargo:rustc-link-lib={}=ui",
+        if cfg!(feature = "build") {
+            "static"
+        } else {
+            "dylib"
+        },
+    );
 
     bindings::generate(&libui_dir, &out_dir).map_err(Error::GenBindings)?;
 
@@ -61,6 +73,39 @@ fn main() -> Result<(), Error> {
 }
 
 #[cfg(feature = "build")]
+fn import_dylibs() {
+    macro_rules! dyn_link {
+        ($($name:tt)*) => {
+            $(
+                println!("cargo:rustc-link-lib=dylib={}", stringify!($name));
+            )*
+        };
+    }
+
+    if build_cfg!(target_os = "linux") {
+        // While unintuitive, we don't actually need to specify any shared objects here---the
+        // `pkg_config` crate will do that automatically in [`bindings::ClangArgs::new_unix`].
+    } else if build_cfg!(target_os = "windows") {
+        // See `dep/libui-ng/windows/meson.build`.
+        dyn_link! {
+            comctl32
+            comdlg32
+            d2d1
+            dwrite
+            gdi32
+            kernel32
+            msimg32
+            ole32
+            oleacc
+            oleaut32
+            user32
+            uuid
+            uxtheme
+            windowscodecs
+        };
+    }
+}
+
 mod dep {
     use std::path::Path;
 
@@ -84,7 +129,7 @@ mod dep {
 
 #[cfg(feature = "build")]
 mod libui {
-    use std::path::Path;
+    use std::{io, path::Path};
 
     /// The error type returned by *libui* functions.
     #[derive(Debug)]
@@ -97,6 +142,10 @@ mod libui {
         BuildLibuiWithNinja(crate::ninja::Error),
         /// Failed to build *libui* with Meson.
         BuildLibuiWithMeson(crate::meson::Error),
+        /// Failed to rename "libui.a" to "ui.lib".
+        ///
+        /// This error *should* only occur when `$CARGO_CFG_TARGET_OS` is `windows`.
+        RenameLibui(io::Error),
     }
 
     /// Builds *libui*.
@@ -104,10 +153,20 @@ mod libui {
         crate::meson::setup_libui(meson_dir, libui_dir).map_err(Error::SetupLibui)?;
         if build_cfg!(target_os = "linux") {
             crate::ninja::build(ninja_dir).map_err(Error::BuildNinja)?;
-            crate::ninja::build_libui(ninja_dir, libui_dir).map_err(Error::BuildLibuiWithNinja)
+            crate::ninja::build_libui(ninja_dir, libui_dir).map_err(Error::BuildLibuiWithNinja)?;
         } else {
-            crate::meson::build_libui(meson_dir, libui_dir).map_err(Error::BuildLibuiWithMeson)
+            crate::meson::build_libui(meson_dir, libui_dir).map_err(Error::BuildLibuiWithMeson)?;
         }
+
+        // Meson unconditionally names the library "libui.a", which prevents MSVC's `link.exe` from
+        // finding it; we must manually rename it to "ui.lib".
+        if build_cfg!(target_os = "windows") {
+            let build_dir = libui_dir.join("build/meson-out");
+            std::fs::rename(build_dir.join("libui.a"), build_dir.join("ui.lib"))
+                .map_err(Error::RenameLibui)?;
+        }
+
+        Ok(())
     }
 }
 
@@ -129,9 +188,11 @@ mod meson {
         let out = process::Command::new("python")
             .arg(meson_dir.join("meson.py"))
             .arg("setup")
-            .arg(format!("--default-library=static"))
-            .arg(format!("--buildtype={}", env::var("PROFILE").unwrap()))
+            .arg("--default-library=static")
+            .arg("--buildtype=release")
+            .arg(format!("--optimization={}", optimization()))
             .arg(format!("--backend={}", backend()))
+            .arg("-Db_vscrt=from_buildtype")
             .arg(libui_dir.join("build"))
             .arg(libui_dir)
             .output()
@@ -156,6 +217,19 @@ mod meson {
         }
     }
 
+    fn is_debug() -> bool {
+        !matches!(env::var("DEBUG").as_deref(), Ok("0" | "false"))
+    }
+
+    fn optimization() -> String {
+        let level = env::var("OPT_LEVEL").expect("$OPT_LEVEL is unset");
+        match level.as_str() {
+            // Meson doesn't support "-Oz"; we'll try the next-closest option.
+            "z" => String::from("s"),
+            _ => level,
+        }
+    }
+
     pub fn build_libui(meson_dir: &Path, libui_dir: &Path) -> Result<(), Error> {
         let out = process::Command::new("python")
             .arg(meson_dir.join("meson.py"))
